#ifdef WIN32
#include <windows.h>
#endif

#ifdef __APPLE__
#include <OpenGL/gl.h>
#include <OpenGL/glu.h>
#else
#include <GL/gl.h>
#include <GL/glu.h>
#endif

#include "OpenGLSceneHandler.h"

OpenGLSceneHandler::OpenGLSceneHandler(): _inited(false) 
{
}

bool OpenGLSceneHandler::getSceneBounds( Producer::Vec3 &center, double &radius )
{
    center.set( 0, 0, 0 );
    radius = 16.0;
    return true;
}

void drawScene()
{
    float ambient_and_diffuse [] = { 0.3f, 0.3f, 0.3f, 1.0f };
    float specular[] = { 0.8f, 0.8f, 0.8f, 1.f };
    float emission[] = { 0.f, 0.f, 0.f, 1.f };

    glMaterialfv( GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, ambient_and_diffuse );
    glMaterialfv( GL_FRONT_AND_BACK, GL_SPECULAR, specular );
    glMaterialfv( GL_FRONT_AND_BACK, GL_EMISSION, emission );
    glMateriali( GL_FRONT_AND_BACK, GL_SHININESS, 64 );
    glutSolidTeapot(12.0);

    const int n = 1;

    for( int i = 0; i < n; i++ )
    {
        glPushMatrix();

        glRotatef( (float)(i * 360/5), 0, 1, 0 );
        glTranslatef( 48.0, 0.0, 0.0 );
        glutSolidTeapot(12.0);
        glPopMatrix();
    }
}

void OpenGLSceneHandler::draw( Producer::Camera &camera) 
{ 
    if( !_inited ) _init();

    drawScene();

    // Wire frame on top of fill
    if( _polymode == GL_FILL + 1 )
    {
        glPushAttrib( GL_ENABLE_BIT| GL_POLYGON_BIT );
        glDisable( GL_LIGHTING );
        glDisable( GL_TEXTURE_2D);
        glEnable( GL_POLYGON_OFFSET_FILL );
        glEnable( GL_POLYGON_OFFSET_LINE );
        glPolygonOffset( -1.0, -1.0 );
        glPolygonMode( GL_FRONT_AND_BACK, GL_LINE );
        drawScene();
        glPopAttrib();
    }

    glFlush();    
}

void OpenGLSceneHandler::toggleLighting()
{
    _lighting = !_lighting;
    if( _lighting )
        glEnable( GL_LIGHTING );
    else
        glDisable( GL_LIGHTING );
}

void OpenGLSceneHandler::toggleTexturing()
{
    _texturing = !_texturing;
    if( _texturing )
        glEnable( GL_TEXTURE_2D );
    else
        glDisable( GL_TEXTURE_2D );
}

void OpenGLSceneHandler::toggleBackfaceCulling()
{
    _backface = !_backface;
    if( _backface )
        glEnable( GL_CULL_FACE );
    else
        glDisable( GL_CULL_FACE );
}

void OpenGLSceneHandler::toggleWireframe()
{
    _polymode = 
        _polymode == GL_FILL ? GL_LINE :
        _polymode == GL_LINE ? GL_POINT :
        _polymode == GL_POINT ? GL_FILL + 1 :
        _polymode == (GL_FILL + 1) ? GL_FILL : GL_FILL;

    if( _polymode == GL_FILL + 1 )
        glPolygonMode( GL_FRONT_AND_BACK, GL_FILL );
    else
        glPolygonMode( GL_FRONT_AND_BACK, _polymode );

}

static void createAndBindCheckboardTexture()
{
    unsigned char *data = new unsigned char[256*256*3];
    unsigned char *ptr = data;
    bool flag = false;
    for( int i = 0; i < 256; i++ )
    {
        if( !(i%16) ) flag = !flag;
        for( int j = 0; j < 256; j++ )
        {
            if( !(j%16) ) flag = !flag;

            if( flag )
            {
                *(ptr++) = 0xFF;
                *(ptr++) = 0xFF;
                *(ptr++) = 0xFF;
            }
            else
            {
                *(ptr++) = 0xFF;
                *(ptr++) = 0x00;
                *(ptr++) = 0x00;
            }
        }
    }
    GLuint tex;
    glGenTextures( 1, &tex );
    glBindTexture( GL_TEXTURE_2D, tex );
    glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
    glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
    glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR );
    gluBuild2DMipmaps( GL_TEXTURE_2D, GL_RGB, 256, 256,  GL_RGB, GL_UNSIGNED_BYTE, data );
    glBindTexture( GL_TEXTURE_2D, 0 );
    glBindTexture( GL_TEXTURE_2D, tex );
}

void OpenGLSceneHandler::_init()
{
    if( _inited ) return;

     glEnable( GL_DEPTH_TEST );

     _polymode =  GL_FILL;

    _backface = true;
    glEnable( GL_CULL_FACE );
    glCullFace( GL_FRONT );

    _lighting = true;
     glEnable( GL_LIGHTING );
     glEnable( GL_LIGHT0 );

     _texturing = true;
     glEnable( GL_TEXTURE_2D );

     createAndBindCheckboardTexture();

    _inited = true;
}
